@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+
 import argparse
 import os
 import sys
@@ -13,34 +14,25 @@ try:
 except ImportError:
     TKINTER = False
 
-from chgksuite.parser import gui_parse
-from chgksuite.composer import gui_compose
-from chgksuite.trello import gui_trello
-from chgksuite.version import __version__
-from chgksuite.common import (
-    on_close,
-    button_factory,
-    toggle_factory,
-    DefaultNamespace,
-    bring_to_front,
-    get_lastdir,
-    ensure_utf8,
-    get_source_dirs,
-)
-
-from collections import defaultdict
 import json
 import shlex
 
-LANGS = ["ru", "ua", "by", "en", "uz", "uz_cyr", "custom"]
+from chgksuite.common import (
+    DefaultNamespace,
+    ensure_utf8,
+    get_lastdir,
+    get_source_dirs,
+)
+from chgksuite.composer import gui_compose
+from chgksuite.parser import gui_parse
+from chgksuite.trello import gui_trello
+from chgksuite.version import __version__
 
-try:
-    basestring
-except NameError:
-    basestring = (str, bytes)
+LANGS = ["by", "by_tar", "en", "kz_cyr", "ru", "sr", "ua", "uz", "uz_cyr"] + ["custom"]
 
 debug = False
 
+
 class VarWrapper(object):
     def __init__(self, name, var):
         self.name = name
@@ -182,7 +174,6 @@ class ParserWrapper(object):
         self.advanced_frame.pack_forget()
 
     def add_argument(self, *args, **kwargs):
-
         if kwargs.pop("advanced", False):
             frame = self.advanced_frame
         else:
@@ -369,12 +360,21 @@ class ArgparseBuilder:
         )
         self.add_argument(
             cmdparse,
-            "--language", "-lang",
+            "--language",
+            "-lang",
             help="language to use while parsing.",
             choices=LANGS,
             default="ru",
             caption="Язык",
-            argtype="radiobutton"
+            argtype="radiobutton",
+        )
+        self.add_argument(
+            cmdparse,
+            "--labels_file",
+            help="i18n config",
+            caption="Конфиг для интернационализации",
+            advanced=True,
+            argtype="filename",
         )
         self.add_argument(
             cmdparse,
@@ -384,6 +384,15 @@ class ArgparseBuilder:
             advanced=True,
             caption="Дописать отсутствующего автора из имени файла",
         )
+        self.add_argument(
+            cmdparse,
+            "--preserve_formatting",
+            "-pf",
+            action="store_true",
+            help="Preserve bold and italic.",
+            caption="Сохранять полужирный и курсив",
+            advanced=True,
+        )
         self.add_argument(
             cmdparse,
             "--encoding",
@@ -432,16 +441,22 @@ class ArgparseBuilder:
         )
         self.add_argument(
             cmdparse,
-            "--image_prefix",
+            "--no_image_prefix",
             action="store_true",
-            help="make image prefix from filename",
+            help="don't make image prefix from filename",
             advanced=True,
-            caption="Make image prefix from filename",
+            caption="Don't make image prefix from filename",
         )
         self.add_argument(
             cmdparse,
             "--parsing_engine",
-            choices=["pypandoc", "pypandoc_html", "mammoth_bs_prettify", "mammoth_bs_hard_unwrap", "mammoth"],
+            choices=[
+                "pypandoc",
+                "pypandoc_html",
+                "mammoth_bs_prettify",
+                "mammoth_bs_hard_unwrap",
+                "mammoth",
+            ],
             default="mammoth",
             help="old html processing behaviour (before v0.5.5). "
             "Sometimes it will yield better results than the new default.",
@@ -465,19 +480,21 @@ class ArgparseBuilder:
             cmdparse,
             "--typography_quotes",
             default="on",
-            choices=["on", "off"],
+            choices=["smart", "on", "off"],
             help="typography: try to fix quotes.",
             advanced=True,
             caption="Типография: кавычки",
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdparse,
             "--typography_dashes",
             default="on",
-            choices=["on", "off"],
+            choices=["smart", "on", "off"],
             help="typography: try to fix dashes.",
             advanced=True,
             caption="Типография: тире",
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdparse,
@@ -487,15 +504,17 @@ class ArgparseBuilder:
             help="typography: try to fix whitespace.",
             advanced=True,
             caption="Типография: whitespace",
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdparse,
             "--typography_accents",
             default="on",
-            choices=["on", "off"],
+            choices=["smart", "light", "on", "off"],
             help="typography: try to fix accents.",
             advanced=True,
             caption="Типография: ударения",
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdparse,
@@ -505,6 +524,17 @@ class ArgparseBuilder:
             help="typography: try to fix percent encoding.",
             advanced=True,
             caption="Типография: %-энкодинг ссылок",
+            argtype="radiobutton",
+        )
+        self.add_argument(
+            cmdparse,
+            "--single_number_line_handling",
+            default="smart",
+            choices=["smart", "on", "off"],
+            help="handling cases where a line consists of a single number.",
+            advanced=True,
+            caption="Обработка строчек, состоящих из одного числа",
+            argtype="radiobutton",
         )
 
         cmdcompose = subparsers.add_parser("compose")
@@ -518,20 +548,13 @@ class ArgparseBuilder:
         )
         self.add_argument(
             cmdcompose,
-            "--language", "-lang",
+            "--language",
+            "-lang",
             help="language to use while composing.",
             choices=LANGS,
             default="ru",
             caption="Язык",
-            argtype="radiobutton"
-        )
-        self.add_argument(
-            cmdcompose,
-            "--nots",
-            action="store_true",
-            help="don't append timestamp to filenames",
-            caption="Не добавлять временную отметку в имя файла",
-            advanced=True,
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdcompose,
@@ -541,6 +564,15 @@ class ArgparseBuilder:
             advanced=True,
             argtype="filename",
         )
+        self.add_argument(
+            cmdcompose,
+            "--add_ts",
+            "-ts",
+            action="store_true",
+            help="append timestamp to filenames",
+            caption="Добавить временную отметку в имя файла",
+            advanced=True,
+        )
         self.add_argument(
             cmdcompose,
             "--imgur_client_id",
@@ -568,19 +600,23 @@ class ArgparseBuilder:
         )
         self.add_argument(
             cmdcompose_docx,
-            "--nospoilers",
-            "-n",
-            action="store_true",
-            help="do not whiten (spoiler) answers.",
-            caption="Не забелять ответы",
+            "--spoilers",
+            "-s",
+            choices=["off", "whiten", "pagebreak", "dots"],
+            default="off",
+            help="whether to hide answers behind spoilers.",
+            caption="Спойлеры",
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdcompose_docx,
-            "--pagebreakspoilers",
-            "-pbs",
-            action="store_true",
-            help="use page break spoilers instead of whiten spoilers.",
-            caption="Использовать новую страницу, а не забеление в качестве защиты от спойлеров",
+            "--screen_mode",
+            "-sm",
+            choices=["off", "replace_all", "add_versions"],
+            default="off",
+            help="exporting questions for screen.",
+            caption="Экспорт для экрана",
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdcompose_docx,
@@ -747,6 +783,14 @@ class ArgparseBuilder:
             caption="Имя 4s-файла",
             filetypes=[("chgksuite markup files", "*.4s")],
         )
+        self.add_argument(
+            cmdcompose_pptx,
+            "--disable_numbers",
+            help="do not put question numbers on slides.",
+            advanced=False,
+            caption="Не добавлять номера вопросов",
+            action="store_true",
+        )
         self.add_argument(
             cmdcompose_pptx,
             "--pptx_config",
@@ -784,14 +828,14 @@ class ArgparseBuilder:
             "--tgchannel",
             required=True,
             help="a channel to post questions to.",
-            caption="Название канала, в который постим"
+            caption="Название канала, в который постим",
         )
         self.add_argument(
             cmdcompose_telegram,
             "--tgchat",
             required=True,
             help="a chat connected to the channel.",
-            caption="Название чата, привязанного к каналу"
+            caption="Название чата, привязанного к каналу",
         )
         self.add_argument(
             cmdcompose_telegram,
@@ -799,7 +843,23 @@ class ArgparseBuilder:
             advanced=True,
             action="store_true",
             help="don't try to post.",
-            caption="Тестовый прогон (не постить в телеграм, только подключиться)"
+            caption="Тестовый прогон (не постить в телеграм, только подключиться)",
+        )
+        self.add_argument(
+            cmdcompose_telegram,
+            "--reset_api",
+            advanced=True,
+            action="store_true",
+            help="reset api_id/api_hash.",
+            caption="Сбросить api_id/api_hash",
+        )
+        self.add_argument(
+            cmdcompose_telegram,
+            "--no_hide_password",
+            advanced=True,
+            action="store_true",
+            help="don't hide 2FA password.",
+            caption="Не скрывать пароль 2FA при вводе (включите, если в вашем терминале есть проблемы со вводом пароля)",
         )
         self.add_argument(
             cmdcompose_telegram,
@@ -814,7 +874,7 @@ class ArgparseBuilder:
             "--skip_until",
             type=int,
             help="skip questions until N.",
-            caption="Пропустить вопросы до N-го",
+            caption="Начать выкладывать с N-го вопроса",
         )
         self.add_argument(
             cmdcompose_telegram,
@@ -839,11 +899,26 @@ class ArgparseBuilder:
             help="tournament id (comma-separated in case of sync+async parts).",
             caption="id турнира (через запятую для синхрона+асинхрона)",
         )
+        self.add_argument(
+            cmdcompose_add_stats,
+            "--custom_csv",
+            help="custom csv/xlsx in rating.chgk.info format",
+            caption="кастомный csv/xlsx с результатами в формате rating.chgk.info",
+            argtype="filename",
+        )
+        self.add_argument(
+            cmdcompose_add_stats,
+            "--custom_csv_args",
+            help="""custom csv arguments in json format (e.g. {"delimiter": ";"})""",
+            default="{}",
+            caption="""кастомные параметры для импорта csv (например, {"delimiter": ";"})""",
+            advanced=True,
+        )
         self.add_argument(
             cmdcompose_add_stats,
             "--question_range",
             help="range of question numbers to include.",
-            caption="Диапазон вопросов (например, \"25-36\"), по умолчанию берутся все)",
+            caption='Диапазон вопросов (например, "25-36"), по умолчанию берутся все)',
         )
         self.add_argument(
             cmdcompose_add_stats,
@@ -854,7 +929,15 @@ class ArgparseBuilder:
             help="threshold for naming teams who scored at the question.",
             caption="Граница вывода названий команд",
         )
-
+        cmdcompose_openquiz = cmdcompose_filetype.add_parser("openquiz")
+        self.add_argument(
+            cmdcompose_openquiz,
+            "filename",
+            nargs="*",
+            help="file(s) to compose from.",
+            caption="Имя 4s-файла",
+            filetypes=[("chgksuite markup files", "*.4s")],
+        )
 
         cmdtrello = subparsers.add_parser("trello")
         cmdtrello_subcommands = cmdtrello.add_subparsers(dest="trellosubcommand")
@@ -867,6 +950,12 @@ class ArgparseBuilder:
             help="path to the folder" "to synchronize with a trello board.",
             caption="Папка",
         )
+        self.add_argument(
+            cmdtrello_download,
+            "--lists",
+            help="Download only specified lists.",
+            caption="Скачать только указанные списки (через запятую)",
+        )
         self.add_argument(
             cmdtrello_download,
             "--si",
@@ -889,10 +978,12 @@ class ArgparseBuilder:
             cmdtrello_download,
             "--fix_trello_new_editor",
             "-ftne",
-            action="store_true",
+            choices=["on", "off"],
+            default="on",
             help="This flag fixes mess caused by Trello's new editor "
-                "(introduced in early 2023).",
+            "(introduced in early 2023).",
             caption="Пофиксить новый редактор Трелло",
+            argtype="radiobutton",
         )
         self.add_argument(
             cmdtrello_download,
@@ -954,24 +1045,17 @@ class ArgparseBuilder:
         cmdtrello_subcommands.add_parser("token")
 
 
-def app():
-    sourcedir, resourcedir = get_source_dirs()
-
-    if isinstance(sourcedir, bytes):
-        sourcedir = sourcedir.decode("utf8")
-    ld = get_lastdir()
-    parser = argparse.ArgumentParser(prog="chgksuite")
-    use_wrapper = len(sys.argv) == 1 and TKINTER
+def single_action(args, use_wrapper, resourcedir):
     if use_wrapper:
-        parser = ParserWrapper(parser, lastdir=ld)
-    ArgparseBuilder(parser, use_wrapper).build()
-    args = DefaultNamespace(parser.parse_args())
+        args.console_mode = False
+    else:
+        args.console_mode = True
 
     if args.language in LANGS:
         if args.action == "parse":
-            args.regexes = os.path.join(resourcedir, f"regexes_{args.language}.json")
-        elif args.action == "compose":
-            args.labels_file = os.path.join(resourcedir, f"labels_{args.language}.toml")
+            regex_lang = "by" if args.language == "by_tar" else args.language
+            args.regexes = os.path.join(resourcedir, f"regexes_{regex_lang}.json")
+        args.labels_file = os.path.join(resourcedir, f"labels_{args.language}.toml")
     if not args.docx_template:
         args.docx_template = os.path.join(resourcedir, "template.docx")
     if not args.pptx_config:
@@ -982,7 +1066,7 @@ def app():
         with open(args.config, "r") as f:
             config = json.load(f)
         for key in config:
-            if not isinstance(config[key], basestring):
+            if not isinstance(config[key], str):
                 val = config[key]
             elif os.path.isfile(config[key]):
                 val = os.path.abspath(config[key])
@@ -998,8 +1082,29 @@ def app():
 
     args.passthrough = False
     if args.action == "parse":
-        gui_parse(args, sourcedir=sourcedir)
+        gui_parse(args)
     if args.action == "compose":
-        gui_compose(args, sourcedir=sourcedir)
+        gui_compose(args)
     if args.action == "trello":
-        gui_trello(args, sourcedir=sourcedir)
+        gui_trello(args)
+
+
+def app():
+    sourcedir, resourcedir = get_source_dirs()
+
+    if isinstance(sourcedir, bytes):
+        sourcedir = sourcedir.decode("utf8")
+    ld = get_lastdir()
+    use_wrapper = len(sys.argv) == 1 and TKINTER
+    if use_wrapper:
+        while True:
+            parser = argparse.ArgumentParser(prog="chgksuite")
+            parser = ParserWrapper(parser, lastdir=ld)
+            ArgparseBuilder(parser, use_wrapper).build()
+            args = DefaultNamespace(parser.parse_args())
+            single_action(args, False, resourcedir)
+    else:
+        parser = argparse.ArgumentParser(prog="chgksuite")
+        ArgparseBuilder(parser, use_wrapper).build()
+        args = DefaultNamespace(parser.parse_args())
+        single_action(args, False, resourcedir)
